@@ -7,7 +7,7 @@
 __copyright__ = """
     pySART - Simplified AUTOSAR-Toolkit for Python.
 
-   (C) 2009-2020 by Christoph Schueler <cpu12.gems@googlemail.com>
+   (C) 2009-2021 by Christoph Schueler <cpu12.gems@googlemail.com>
 
    All Rights Reserved
 
@@ -27,21 +27,21 @@ __copyright__ = """
 """
 
 import abc
+from bisect import bisect_left
 from collections import OrderedDict
 import functools
 import operator
 from time import perf_counter, time
 
-from typing import Type
-
 from pyxcp.transport.base import BaseTransport
 from pyxcp.config import Configuration
-from pyxcp.types import NumericType
 
 
-CAN_EXTENDED_ID = 0x80000000
-MAX_11_BIT_IDENTIFIER = (1 << 11) - 1
-MAX_29_BIT_IDENTIFIER = (1 << 29) - 1
+CAN_EXTENDED_ID         = 0x80000000
+MAX_11_BIT_IDENTIFIER   = (1 << 11) - 1
+MAX_29_BIT_IDENTIFIER   = (1 << 29) - 1
+MAX_DLC_CLASSIC         = 8
+CAN_FD_DLCS             = (12, 16, 20, 24, 32, 48, 64)  # Discrete CAN-FD DLCs in case DLC > 8.
 
 
 class IdentifierOutOfRangeError(Exception):
@@ -98,6 +98,26 @@ def samplePointToTsegs(tqs: int, samplePoint: float) -> tuple:
     return (tseg1, tseg2)
 
 
+def padFrame(frame: bytes, padding_value: int, padding_len:int = 0) -> bytes:
+    """Pad frame to next discrete DLC value.
+
+    References:
+    -----------
+    ISO/DIS 15765 - 4; 8.2 Data length Code (DLC)
+    AUTOSAR CP Release 4.3.0, Specification of CAN Transport Layer; 7.3.8 N-PDU padding
+    AUTOSAR CP Release 4.3.0, Specification of CAN Driver; [SWS_CAN_00502], [ECUC_Can_00485]
+    AUTOSAR CP Release 4.3.0, Requirements on CAN; [SRS_Can_01073], [SRS_Can_01086], [SRS_Can_01160]
+    """
+    frame_len = max(len(frame), padding_len)
+    if frame_len <= MAX_DLC_CLASSIC:
+        actual_len = MAX_DLC_CLASSIC
+    else:
+        actual_len = CAN_FD_DLCS[bisect_left(CAN_FD_DLCS, frame_len)]
+    # append fill bytes up to MAX_DLC resp. next discrete FD DLC.
+    if len(frame) < actual_len:
+        frame += bytes([padding_value]) * (actual_len - len(frame))
+    return frame
+
 class Identifier:
     """Convenience class for XCP formatted CAN identifiers.
 
@@ -270,18 +290,20 @@ class Can(BaseTransport):
 
     PARAMETER_MAP = {
         #                           Type            Req'd   Default
-        "CAN_DRIVER": (str, True, None),
-        "CHANNEL": (str, False, ""),
-        "MAX_DLC_REQUIRED": (bool, False, False),
-        "CAN_USE_DEFAULT_LISTENER": (bool, False, True),
+        "CAN_DRIVER":               (str,           True,   None),
+        "CHANNEL":                  (str,           False,  ""),
+        "MAX_DLC_REQUIRED":         (bool,          False,  False),
+        "MAX_CAN_FD_DLC":           (int,           False,  64),
+        "PADDING_VALUE":            (int,           False,   0),
+        "CAN_USE_DEFAULT_LISTENER": (bool,          False,  True),
         # defaults to True, in this case the default listener thread is used.
         # If the canInterface implements a listener service, this parameter
         # can be set to False, and the default listener thread won't be started.
-        "CAN_ID_MASTER": (int, True, None),
-        "CAN_ID_SLAVE": (int, True, None),
-        "CAN_ID_BROADCAST": (int, False, None),
-        "BITRATE": (int, False, 250000),
-        "RECEIVE_OWN_MESSAGES": (bool, False, False),
+        "CAN_ID_MASTER":            (int,           True,   None),
+        "CAN_ID_SLAVE":             (int,           True,   None),
+        "CAN_ID_BROADCAST":         (int,           False,  None),
+        "BITRATE":                  (int,           False,  250000),
+        "RECEIVE_OWN_MESSAGES":     (bool,          False,  False),
     }
 
     PARAMETER_TO_KW_ARG_MAP = {
@@ -309,11 +331,17 @@ class Can(BaseTransport):
         canInterfaceClass = drivers[interfaceName]
         self.canInterface = canInterfaceClass()
         self.useDefaultListener = self.config.get("CAN_USE_DEFAULT_LISTENER")
-        self.max_dlc_required = self.config.get("MAX_DLC_REQUIRED")
         self.can_id_master = Identifier(self.config.get("CAN_ID_MASTER"))
         self.can_id_slave = Identifier(self.config.get("CAN_ID_SLAVE"))
         self.canInterface.loadConfig(config)
         self.canInterface.init(self, self.dataReceived)
+        #
+        # Regarding CAN-FD s. AUTOSAR CP Release 4.3.0, Requirements on CAN; [SRS_Can_01160] Padding of bytes due to discrete CAN FD DLC]:
+        #   "... If a PDU does not exactly match these configurable sizes the unused bytes shall be padded."
+        #
+        self.max_dlc_required = self.config.get("MAX_DLC_REQUIRED") or self.canInterface.is_fd
+        self.padding_value = self.config.get("PADDING_VALUE")
+        self.padding_len = self.config.get("MAX_CAN_FD_DLC") if self.canInterface.is_fd else MAX_DLC_CLASSIC
 
     def dataReceived(self, payload: bytes, recv_timestamp: float = None):
         self.processResponse(payload, len(payload), counter=self.counterReceived + 1, recv_timestamp=recv_timestamp)
@@ -335,17 +363,15 @@ class Can(BaseTransport):
     def send(self, frame):
         # XCP on CAN trailer: if required, FILL bytes must be appended
         if self.max_dlc_required:
-            # append fill bytes up to MAX DLC (=8)
-            if len(frame) < 8:
-                frame += b"\x00" * (8 - len(frame))
+            frame = padFrame(frame, self.padding_value, self.padding_len)
         # send the request
         if self.perf_counter_origin > 0:
             self.pre_send_timestamp = time()
-            self.canInterface.transmit(payload=frame)
+            self.canInterface.transmit(payload = frame)
             self.post_send_timestamp = time()
         else:
             pre_send_timestamp = perf_counter()
-            self.canInterface.transmit(payload=frame)
+            self.canInterface.transmit(payload = frame)
             post_send_timestamp = perf_counter()
             self.pre_send_timestamp = self.timestamp_origin + pre_send_timestamp - self.perf_counter_origin
             self.post_send_timestamp = self.timestamp_origin + post_send_timestamp - self.perf_counter_origin
@@ -354,6 +380,9 @@ class Can(BaseTransport):
         if hasattr(self, "canInterface"):
             self.canInterface.close()
 
+    def close(self):
+        self.closeConnection()
+
 
 def setDLC(length: int):
     """Return DLC value according to CAN-FD.
@@ -361,14 +390,13 @@ def setDLC(length: int):
     :param length: Length value to be mapped to a valid CAN-FD DLC.
                    ( 0 <= length <= 64)
     """
-    FD_DLCS = (12, 16, 20, 24, 32, 48, 64)
 
     if length < 0:
         raise ValueError("Non-negative length value required.")
-    elif length <= 8:
+    elif length <= MAX_DLC_CLASSIC:
         return length
     elif length <= 64:
-        for dlc in FD_DLCS:
+        for dlc in CAN_FD_DLCS:
             if length <= dlc:
                 return dlc
     else:
